@@ -10,9 +10,9 @@ trait.TMB <- function(
       Lambda.struc = "unstructured", Ab.struct = "unstructured", row.eff = FALSE, reltol = 1e-6, seed = NULL,
       maxit = 1000, start.lvs = NULL, offset=NULL, sd.errors = TRUE,trace=FALSE,
       link="logit",n.init=1,start.params=NULL,start0=FALSE,optimizer="optim",
-      starting.val="res",method="VA",randomX=NULL,Power=1.5,diag.iter=1, Ab.diag.iter = 0, dependent.row = TRUE,
+      starting.val="res",method="VA",randomX=NULL,Power=1.5,diag.iter=1, Ab.diag.iter = 0, dependent.row = FALSE,
       Lambda.start=c(0.1, 0.5), jitter.var=0, yXT = NULL, scale.X = FALSE, randomX.start = "zero", beta0com = FALSE
-      ) {
+      ,zeta.struc = "species") {
   if(is.null(X) && !is.null(TR)) stop("Unable to fit a model that includes only trait covariates")
   if(!is.null(start.params)) starting.val <- "zero"
   
@@ -143,12 +143,12 @@ trait.TMB <- function(
     colnames(X1) <- colnames(X.new)[fx]; colnames(TR1)<-colnames(T.new)[ft];
     nxd <- colnames(Xd)
     formulab <- paste("~",nxd[1],sep = "");
-    for(i in 2:length(nxd)) formulab <- paste(formulab,nxd[i],sep = "+")
+    if(length(nxd)>1) for(i in 2:length(nxd)) formulab <- paste(formulab,nxd[i],sep = "+")
     formula1 <- formulab
   }
 
 
-  if(!(family %in% c("poisson","negative.binomial","binomial","tweedie","ZIP", "gaussian", "ordinal")))
+  if(!(family %in% c("poisson","negative.binomial","binomial","tweedie","ZIP", "gaussian", "ordinal", "gamma", "exponential")))
     stop("Selected family not permitted...sorry!")
   if(!(Lambda.struc %in% c("unstructured","diagonal")))
     stop("Lambda matrix (covariance of vartiational distribution for latent variable) not permitted...sorry!")
@@ -163,9 +163,12 @@ trait.TMB <- function(
     max.levels <- apply(y,2,function(x) length(min(x):max(x)))
     if(any(max.levels == 1) || all(max.levels == 2))
       stop("Ordinal data requires all columns to have at least has two levels. If all columns only have two levels, please use family == binomial instead. Thanks")
-                        
-    if(any(!apply(y,2,function(x)all(diff(sort(unique(x)))==1))))
-      stop("Can't fit ordinal model if there are species with missing classes. Please reclassify per species.")
+    
+    if(any(!apply(y,2,function(x)all(diff(sort(unique(x)))==1)))&zeta.struc=="species")
+      stop("Can't fit ordinal model if there are species with missing classes. Please reclassify per species or use zeta.struc = `common` ")
+    
+    if(any(diff(sort(unique(c(y))))!=1)&zeta.struc=="common")
+      stop("Can't fit ordinal model if there are missing classes. Please reclassify.")
   }
   if(is.null(rownames(y))) rownames(y) <- paste("Row",1:n,sep="")
   if(is.null(colnames(y))) colnames(y) <- paste("Col",1:p,sep="")
@@ -185,7 +188,9 @@ trait.TMB <- function(
     phi <- phis <- NULL;
     
     if(n.init > 1 && trace) cat("initial run ",n.i,"\n");
-    res <- start.values.gllvm.TMB(y = y, X = X1, TR = TR1, family = family, offset=offset, trial.size = trial.size, num.lv = num.lv, start.lvs = start.lvs, seed = seed[n.i],starting.val=starting.val,power=Power,formula = formula, jitter.var=0,yXT=yXT, row.eff = row.eff, TMB=TRUE, link=link, randomX=randomXb, beta0com = beta0com0)
+
+    res <- start.values.gllvm.TMB(y = y, X = X1, TR = TR1, family = family, offset=offset, trial.size = trial.size, num.lv = num.lv, start.lvs = start.lvs, seed = seed[n.i],starting.val=starting.val,power=Power,formula = formula, jitter.var=0,yXT=yXT, row.eff = row.eff, TMB=TRUE, link=link, randomX=randomXb, beta0com = beta0com0, zeta.struc = zeta.struc)
+
     if(is.null(start.params)){
       
       beta0 <- res$params[,1]
@@ -205,7 +210,7 @@ trait.TMB <- function(
       vameans <- theta <- lambda <- NULL
 
       if(num.lv > 0) {
-        if(!is.null(randomXb)){
+        if(!is.null(randomXb) && family != "ordinal"){
           Br <- res$Br
           sigmaB <- (res$sigmaB)
           if(length(sigmaB)>1) sigmaij <- rep(0,length(res$sigmaij))
@@ -285,13 +290,18 @@ trait.TMB <- function(
       phis <- (colMeans(y == 0) * 0.98) + 0.01; 
       phis <- phis / (1 - phis)
       } # ZIP probability
-    # if (family == "gaussian") {
-    #   phis <- fit$phi
+    # if (family %in% c("gaussian", "gamma")) {
+    #   phis <- res$phi
     # }
     if(family=="ordinal"){
       K = max(y00)-min(y00)
-      zeta <- c(t(res$zeta[,-1]))
-      zeta <- zeta[!is.na(zeta)]
+      if(zeta.struc=="species"){
+        zeta <- c(t(res$zeta[,-1]))
+        zeta <- zeta[!is.na(zeta)]
+      }else{
+        zeta <- res$zeta[-1]
+      }
+      
     }else{
       zeta = 0
     }
@@ -353,9 +363,11 @@ trait.TMB <- function(
       if(link=="probit") extra[1] <- 1
     }
     if(family == "gaussian") {familyn=3}
-    if(family == "tweedie"){ familyn <- 4; extra[1] <- Power}
-    if(family == "ZIP"){ familyn <- 5;}
-    if(family == "ordinal") {familyn=6}
+    if(family == "gamma") {familyn=4}
+    if(family == "tweedie"){ familyn <- 5; extra[1] <- Power}
+    if(family == "ZIP"){ familyn <- 6;}
+    if(family == "ordinal") {familyn=7}
+    if(family == "exponential") {familyn=8}
     if(beta0com){
       extra[2] <- 0
       Xd<-cbind(1,Xd)
@@ -386,13 +398,15 @@ trait.TMB <- function(
 
         if(method == "VA"){
           objr <- TMB::MakeADFun(
-            data = list(y = y, x = Xd, xr=xr, xb=xb, offset=offset, num_lv = num.lv, family=familyn, extra=extra, method=0, model=1, random=randoml), silent=!trace,
+
+            data = list(y = y, x = Xd, xr=xr, xb=xb, offset=offset, num_lv = num.lv, family=familyn, extra=extra, method=0, model=1, random=randoml, zetastruc = ifelse(zeta.struc=="species",1,0)), silent=!trace,
             parameters = list(r0=matrix(r0), b = rbind(a), B=matrix(B), Br=Br, lambda = theta, u = u, lg_phi=log(phi), sigmaB=log(sqrt(diag(sigmaB))), sigmaij=sigmaij, log_sigma=c(sigma), Au=Au, Abb=Abb, zeta=zeta),
             inner.control=list(mgcmax = 1e+200,maxit = maxit),
             DLL = "gllvm")
         } else {
           objr <- TMB::MakeADFun(
-            data = list(y = y, x = Xd,xr=xr, xb=xb, offset=offset, num_lv = num.lv, family=familyn,extra=extra,method=1,model=1,random=randoml), silent=!trace,
+
+            data = list(y = y, x = Xd,xr=xr, xb=xb, offset=offset, num_lv = num.lv, family=familyn,extra=extra,method=1,model=1,random=randoml, zetastruc = ifelse(zeta.struc=="species",1,0)), silent=!trace,
             parameters = list(r0=matrix(r0), b = rbind(a),B=matrix(B), Br=Br,lambda = theta, u = u, lg_phi=log(phi), sigmaB=log(sqrt(diag(sigmaB))), sigmaij=sigmaij, log_sigma=c(sigma), Au=0, Abb=0, zeta=zeta),
             inner.control=list(mgcmax = 1e+200,maxit = maxit,tol10=0.01),
             random = c(("Br")[randoml[2]>0],"u"), DLL = "gllvm")
@@ -400,13 +414,13 @@ trait.TMB <- function(
       } else {
         if(method=="VA"){
           objr <- TMB::MakeADFun(
-            data = list(y = y, x = Xd, xr=xr, xb=xb, offset=offset, num_lv = num.lv, family=familyn, extra=extra, method=0, model=1, random=randoml), silent=!trace,
+            data = list(y = y, x = Xd, xr=xr, xb=xb, offset=offset, num_lv = num.lv, family=familyn, extra=extra, method=0, model=1, random=randoml, zetastruc = ifelse(zeta.struc=="species",1,0)), silent=!trace,
             parameters = list(r0=matrix(r0), b = rbind(a), B=matrix(B), Br=Br,lambda = 0, u = u, lg_phi=log(phi), sigmaB=log(sqrt(diag(sigmaB))), sigmaij=sigmaij, log_sigma=c(sigma), Au=Au, Abb=Abb, zeta=zeta),
             inner.control=list(mgcmax = 1e+200,maxit = maxit,tol10=0.01),
             DLL = "gllvm")
         } else {
           objr <- TMB::MakeADFun(
-            data = list(y = y, x = Xd, xr=xr, xb=xb, offset=offset, num_lv = num.lv, family=familyn, extra=extra, method=1, model=1, random=randoml), silent=!trace,
+            data = list(y = y, x = Xd, xr=xr, xb=xb, offset=offset, num_lv = num.lv, family=familyn, extra=extra, method=1, model=1, random=randoml, zetastruc = ifelse(zeta.struc=="species",1,0)), silent=!trace,
             parameters = list(r0=matrix(r0), b = rbind(a), B=matrix(B), Br=Br, lambda = 0, u = u, lg_phi=log(phi), sigmaB=log(sqrt(diag(sigmaB))), sigmaij=sigmaij, log_sigma=c(sigma), Au=0, Abb=0, zeta=zeta),
             inner.control=list(mgcmax = 1e+200,maxit = maxit,tol10=0.01),
             random = c(randoms[randoml>0]), DLL = "gllvm")
@@ -419,20 +433,20 @@ trait.TMB <- function(
       if(num.lv>0) {
         if(method=="VA"){
           objr <- TMB::MakeADFun(
-            data = list(y = y, x = Xd, xr=xr, xb=xb, offset=offset, num_lv = num.lv, family=familyn,extra=extra,method=0,model=1,random=randoml), silent=!trace,
+            data = list(y = y, x = Xd, xr=xr, xb=xb, offset=offset, num_lv = num.lv, family=familyn,extra=extra,method=0,model=1,random=randoml, zetastruc = ifelse(zeta.struc=="species",1,0)), silent=!trace,
             parameters = list(r0=matrix(r0), b = rbind(a),B=matrix(B), Br=Br,lambda = theta, u = u,lg_phi=log(phi), sigmaB=log(sqrt(diag(sigmaB))), sigmaij=sigmaij,log_sigma=0,Au=Au, Abb=0, zeta=zeta),
             inner.control=list(mgcmax = 1e+200,maxit = 1000),
             DLL = "gllvm")
         } else {
           objr <- TMB::MakeADFun(
-            data = list(y = y, x = Xd, xr=xr, xb=xb, offset=offset, num_lv = num.lv, family=familyn,extra=extra,method=1,model=1,random=randoml), silent=!trace,
+            data = list(y = y, x = Xd, xr=xr, xb=xb, offset=offset, num_lv = num.lv, family=familyn,extra=extra,method=1,model=1,random=randoml, zetastruc = ifelse(zeta.struc=="species",1,0)), silent=!trace,
             parameters = list(r0=matrix(r0), b = rbind(a),B=matrix(B), Br=Br,lambda = theta, u = u,lg_phi=log(phi), sigmaB=log(sqrt(diag(sigmaB))), sigmaij=sigmaij,log_sigma=0,Au=0, Abb=0, zeta=zeta),
             inner.control=list(mgcmax = 1e+200,maxit = 1000,tol10=0.01),
             random = c("u"), DLL = "gllvm")
         }
       } else {
         objr <- TMB::MakeADFun(
-          data = list(y = y, x = Xd, xr=xr, xb=xb, offset=offset, num_lv = num.lv, family=familyn,extra=extra,method=1,model=1,random=randoml), silent=!trace,
+          data = list(y = y, x = Xd, xr=xr, xb=xb, offset=offset, num_lv = num.lv, family=familyn,extra=extra,method=1,model=1,random=randoml, zetastruc = ifelse(zeta.struc=="species",1,0)), silent=!trace,
           parameters = list(r0=matrix(r0), b = rbind(a),B=matrix(B), Br=Br,lambda = 0, u = matrix(0),lg_phi=log(phi), sigmaB=log(sqrt(diag(sigmaB))), sigmaij=sigmaij,log_sigma=0,Au=0, Abb=0, zeta=zeta),
           inner.control=list(mgcmax = 1e+200,maxit = 1000,tol10=0.01),
           DLL = "gllvm")
@@ -464,7 +478,7 @@ trait.TMB <- function(
       sigmaij1 <- param1[nam=="sigmaij"]
       lg_sigma1 <- param1[nam=="log_sigma"]
       
-      Au1<- c(pmax(param1[nam=="Au"],rep(log(1e-4), nlvr*n)), rep(0,nlvr*(nlvr-1)/2*n))
+      Au1<- c(pmax(param1[nam=="Au"],rep(log(1e-6), nlvr*n)), rep(0,nlvr*(nlvr-1)/2*n))
       Abb1 <- param1[nam=="Abb"]
       if(Ab.diag.iter>0 && Ab.struct == "unstructured")
         Abb1 <- c(Abb1 + 0.1, rep(0,ncol(xb)*(ncol(xb)-1)/2*p))
@@ -474,13 +488,13 @@ trait.TMB <- function(
 
         if(nlvr>0){
           objr <- TMB::MakeADFun(
-            data = list(y = y, x = Xd, xr=xr, xb=xb, offset=offset, num_lv = num.lv, family=familyn, extra=extra, method=0, model=1, random=randoml), silent=!trace,
+            data = list(y = y, x = Xd, xr=xr, xb=xb, offset=offset, num_lv = num.lv, family=familyn, extra=extra, method=0, model=1, random=randoml, zetastruc = ifelse(zeta.struc=="species",1,0)), silent=!trace,
             parameters = list(r0=r1, b = b1, B=B1, Br=Br1, lambda = lambda1, u = u1, lg_phi=lg_phi1, sigmaB=sigmaB1, sigmaij=sigmaij1, log_sigma=lg_sigma1, Au=Au1, Abb=Abb1, zeta=zeta),
             inner.control=list(mgcmax = 1e+200,maxit = 1000),
             DLL = "gllvm")
         } else {
           objr <- TMB::MakeADFun(
-            data = list(y = y, x = Xd,xr=xr,xb=xb,offset=offset, num_lv = num.lv,family=familyn,extra=extra,method=0,model=1,random=randoml), silent=TRUE,
+            data = list(y = y, x = Xd,xr=xr,xb=xb,offset=offset, num_lv = num.lv,family=familyn,extra=extra,method=0,model=1,random=randoml, zetastruc = ifelse(zeta.struc=="species",1,0)), silent=TRUE,
             parameters = list(r0=r1, b = b1,B=B1,Br=Br1,lambda = 0, u = matrix(0),lg_phi=lg_phi1,sigmaB=sigmaB1,sigmaij=sigmaij1,log_sigma=lg_sigma1,Au=0, Abb=Abb1, zeta=zeta),
             inner.control=list(mgcmax = 1e+200,maxit = maxit,tol10=0.01),
             DLL = "gllvm")
@@ -489,7 +503,7 @@ trait.TMB <- function(
 
       } else {
         objr <- TMB::MakeADFun(
-          data = list(y = y, x = Xd,xr=xr,xb=xb,offset=offset, num_lv = num.lv,family=familyn,extra=extra,method=0,model=1,random=randoml), silent=!trace,
+          data = list(y = y, x = Xd,xr=xr,xb=xb,offset=offset, num_lv = num.lv,family=familyn,extra=extra,method=0,model=1,random=randoml, zetastruc = ifelse(zeta.struc=="species",1,0)), silent=!trace,
           parameters = list(r0=r1, b = b1,B=B1,Br=Br1,lambda = lambda1, u = u1,lg_phi=lg_phi1, sigmaB=sigmaB1, sigmaij=sigmaij1, log_sigma=0, Au=Au1, Abb=Abb1, zeta=zeta),
           inner.control=list(mgcmax = 1e+200,maxit = 1000),
           DLL = "gllvm")
@@ -506,7 +520,7 @@ trait.TMB <- function(
     }
 
     param <- objr$env$last.par.best
-    if(family %in% c("negative.binomial", "tweedie", "gaussian")) {
+    if(family %in% c("negative.binomial", "tweedie", "gaussian", "gamma")) {
       phis=exp(param[names(param)=="lg_phi"])
     }
     if(family=="ZIP") {
@@ -515,19 +529,25 @@ trait.TMB <- function(
     }
     if(family == "ordinal"){
       zetas <- param[names(param)=="zeta"]
-      zetanew <- matrix(NA,nrow=p,ncol=K)
-      idx<-0
-      for(j in 1:ncol(y)){
-        k<-max(y[,j])-2
-        if(k>0){
-          for(l in 1:k){
-            zetanew[j,l+1]<-zetas[idx+l]
-          } 
+      if(zeta.struc=="species"){
+        zetanew <- matrix(NA,nrow=p,ncol=K)
+        idx<-0
+        for(j in 1:ncol(y)){
+          k<-max(y[,j])-2
+          if(k>0){
+            for(l in 1:k){
+              zetanew[j,l+1]<-zetas[idx+l]
+            } 
+          }
+          idx<-idx+k
         }
-        idx<-idx+k
+        zetanew[,1] <- 0 
+        row.names(zetanew) <- colnames(y00); colnames(zetanew) <- paste(min(y):(max(y00)-1),"|",(min(y00)+1):max(y00),sep="")
+      }else{
+        zetanew <- c(0,zetas)
+        names(zetanew) <- paste(min(y00):(max(y00)-1),"|",(min(y00)+1):max(y00),sep="")
       }
-      zetanew[,1] <- 0
-      row.names(zetanew) <- colnames(y00); colnames(zetanew) <- paste(min(y):(max(y00)-1),"|",(min(y00)+1):max(y00),sep="")
+      
       zetas<-zetanew
       out$y<-y00
     }
@@ -594,7 +614,8 @@ trait.TMB <- function(
         rownames(out$params$theta) <- colnames(out$y)
       }
       if(!beta0com) names(beta0) <- colnames(out$y); 
-      names(beta0) <- colnames(out$y); out$params$beta0 <- beta0;
+      if(beta0com) names(beta0) <- "Community intercept";
+      out$params$beta0 <- beta0;
       out$params$B <- B; names(out$params$B)=colnames(Xd)
 
       if(row.eff!=FALSE) {
@@ -610,7 +631,7 @@ trait.TMB <- function(
         out$params$phi <- 1/phis; names(out$params$phi) <- colnames(out$y);
         out$params$inv.phi <- phis; names(out$params$inv.phi) <- colnames(out$y);
       }
-      if(family %in% c("gaussian","tweedie")) {
+      if(family %in% c("gaussian","tweedie","gamma")) {
         out$params$phi <- phis; names(out$params$phi) <- colnames(out$y);
       }
       if(family =="ZIP") {
@@ -721,9 +742,9 @@ trait.TMB <- function(
         incl[names(objrFinal$par)=="Au"] <- FALSE; if(num.lv>0) incld[names(objrFinal$par)=="Au"] <- TRUE
         incl[names(objrFinal$par)=="u"] <- FALSE; incld[names(objrFinal$par)=="u"] <- TRUE
         
-        if(familyn==0 || familyn==2 || familyn==6) incl[names(objrFinal$par)=="lg_phi"] <- FALSE
-        if(familyn!=6) incl[names(objrFinal$par)=="zeta"] <- FALSE
-        if(familyn==6) incl[names(objrFinal$par)=="zeta"] <- TRUE
+        if(familyn==0 || familyn==2 || familyn==7 || familyn==8) incl[names(objrFinal$par)=="lg_phi"] <- FALSE
+        if(familyn!=7) incl[names(objrFinal$par)=="zeta"] <- FALSE
+        if(familyn==7) incl[names(objrFinal$par)=="zeta"] <- TRUE
         
         if(nlvr==0){
           incl[names(objrFinal$par)=="u"] <- FALSE;
@@ -758,10 +779,10 @@ trait.TMB <- function(
             out$prediction.errors <- prediction.errors
           }
         } else {
-          A.mat <- -sdr[incl,incl] # a x a
-          D.mat <- -sdr[incld,incld] # d x d
-          B.mat <- -sdr[incl,incld] # a x d
-          cov.mat.mod <- try(MASS::ginv(A.mat-B.mat%*%solve(D.mat)%*%t(B.mat)))
+          A.mat <- sdr[incl,incl] # a x a
+          D.mat <- sdr[incld,incld] # d x d
+          B.mat <- sdr[incl,incld] # a x d
+          cov.mat.mod <- try(MASS::ginv(A.mat-B.mat%*%solve(D.mat)%*%t(B.mat)),silent=T) 
           se <- sqrt(diag(abs(cov.mat.mod)))
           
           incla<-rep(FALSE, length(incl))
@@ -797,7 +818,7 @@ trait.TMB <- function(
           out$sd$phi <- se.lphis*out$params$phi;
           names(out$sd$inv.phi) <- names(out$sd$phi) <- colnames(y);  se <- se[-(1:p)]
         }
-        if(family %in% c("gaussian","tweedie")) {
+        if(family %in% c("gaussian","tweedie","gamma")) {
           se.lphis <- se[1:p];
           out$sd$phi <- se.lphis*out$params$phi;
           names(out$sd$phi) <- colnames(y);  se <- se[-(1:p)]
@@ -807,43 +828,48 @@ trait.TMB <- function(
           out$sd$phi <- se.phis*exp(lp0)/(1+exp(lp0))^2;#
           names(out$sd$phi) <- colnames(y);  se <- se[-(1:p)]
         }
+        if(!is.null(randomX)){
+          nr <- ncol(xb)
+          out$sd$sigmaB <- se[1:ncol(xb)]*c(sqrt(diag(out$params$sigmaB))); 
+          names(out$sd$sigmaB) <- c(paste("sd",colnames(xb),sep = "."))
+          se <- se[-(1:ncol(xb))]
+          if(nr>1){
+            out$sd$corrpar <- se[1:(nr*(nr-1)/2)]
+            se <- se[-(1:(nr*(nr-1)/2))]
+          }
+        }
         if(row.eff=="random") { 
           out$sd$sigma <- se[1:length(out$params$sigma)]*c(out$params$sigma[1],rep(1,length(out$params$sigma)-1)); 
           names(out$sd$sigma) <- "sigma"; 
-          se=se[-(1:(1+num.lv))] 
+          se=se[-(1:(length(out$params$sigma)))] 
           }
-        if(!is.null(randomX)){
-          nr <- ncol(xb)
-          #out$sd$sigmaB <- se[1:ncol(xb)]*c(diag(out$params$sigmaB), rep(1,nr*(nr-1)/2 ) )
-          out$sd$sigmaB <- se[1:ncol(xb)]*c(sqrt(diag(out$params$sigmaB))); 
-          names(out$sd$sigmaB) <- c(paste("sd",colnames(xb),sep = "."))
-          if(length(se)>1){
-            se <- se[-(1:ncol(xb))]
-            out$sd$corrpar <- se
-          }
-          
-        }
         if(family %in% c("ordinal")){
-          se.zetas <- se;
-          se.zetanew <- matrix(NA,nrow=p,ncol=K)
-          idx<-0
-          for(j in 1:ncol(y)){
-            k<-max(y[,j])-2
-            if(k>0){
-              for(l in 1:k){
-                se.zetanew[j,l+1]<-se.zetas[idx+l]
-              } 
+          se.zetanew <- se.zetas <- se;
+          if(zeta.struc == "species"){
+            se.zetanew <- matrix(NA,nrow=p,ncol=K)
+            idx<-0
+            for(j in 1:ncol(y)){
+              k<-max(y[,j])-2
+              if(k>0){
+                for(l in 1:k){
+                  se.zetanew[j,l+1]<-se.zetas[idx+l]
+                } 
+              }
+              idx<-idx+k
             }
-            idx<-idx+k
+            se.zetanew[,1] <- 0
+            out$sd$zeta <- se.zetanew
+            row.names(out$sd$zeta) <- colnames(y00); colnames(out$sd$zeta) <- paste(min(y00):(max(y00)-1),"|",(min(y00)+1):max(y00),sep="")
+            
+          }else{
+            se.zetanew <- c(0, se.zetanew)
+            out$sd$zeta <- se.zetanew
+            names(out$sd$zeta) <- paste(min(y00):(max(y00)-1),"|",(min(y00)+1):max(y00),sep="")
+            
           }
-          se.zetanew[,1] <- 0
-          out$sd$zeta <- se.zetanew
-          row.names(out$sd$zeta) <- colnames(y00); colnames(out$sd$zeta) <- paste(min(y00):(max(y00)-1),"|",(min(y00)+1):max(y00),sep="")
         }
-
+        
       }})
-
-
   if(inherits(tr, "try-error")) { cat("Standard errors for parameters could not be calculated.\n") }
 
   if(is.null(formula1)){ out$formula <- formula} else {out$formula <- formula1}
