@@ -112,8 +112,26 @@ residuals.gllvm <- function(object, ...) {
         ds.res[i, j] <- qnorm(u)
       }
       if (object$family == "gaussian") {
-        a <- pnorm(as.vector(unlist(y[i, j])) - 1, mu[i, j])
-        b <- pnorm(as.vector(unlist(y[i, j])), mu[i, j])
+        phis <- object$params$phi
+        a <- pnorm(as.vector(unlist(y[i, j])), mu[i, j], sd = phis[j])
+        b <- pnorm(as.vector(unlist(y[i, j])), mu[i, j], sd = phis[j])
+        u <- runif(n = 1, min = a, max = b)
+        if(u==1) u=1-1e-16
+        if(u==0) u=1e-16
+        ds.res[i, j] <- qnorm(u)
+      }
+      if (object$family == "gamma") {
+        phis <- object$params$phi # - 1
+        a <- pgamma(as.vector(unlist(y[i, j])), shape = phis[j], scale = mu[i, j]/phis[j])
+        b <- pgamma(as.vector(unlist(y[i, j])), shape = phis[j], scale = mu[i, j]/phis[j])
+        u <- runif(n = 1, min = a, max = b)
+        if(u==1) u=1-1e-16
+        if(u==0) u=1e-16
+        ds.res[i, j] <- qnorm(u)
+      }
+      if (object$family == "exponential") {
+        a <- pexp(as.vector(unlist(y[i, j])), rate = 1/mu[i, j])
+        b <- pexp(as.vector(unlist(y[i, j])), rate = 1/mu[i, j])
         u <- runif(n = 1, min = a, max = b)
         if(u==1) u=1-1e-16
         if(u==0) u=1e-16
@@ -148,28 +166,48 @@ residuals.gllvm <- function(object, ...) {
         ds.res[i, j] <- qnorm(u)
       }
       if (object$family == "ordinal") {
-
-        probK <- NULL
-        probK[1] <- pnorm(object$params$zeta[j, 1] - eta.mat[i, j], log.p = FALSE)
-        probK[max(y[, j]) + 1 - min(y[, j])] <- 1 - pnorm(object$params$zeta[j, max(y[, j]) - min(y[, j])] - eta.mat[i, j])
-        if(max(y[, j]) > 2) {
-          j.levels <- 2:(max(y[, j]) - min(y[, j]))#
-          for (k in j.levels) {
-            probK[k] <- pnorm(object$params$zeta[j, k] - eta.mat[i, j]) - pnorm(object$params$zeta[j, k - 1] - eta.mat[i, j])
+          if(object$zeta.struc == "species"){
+            probK <- NULL
+            probK[1] <- pnorm(object$params$zeta[j, 1] - eta.mat[i, j], log.p = FALSE)
+            probK[max(y[, j]) + 1 - min(y[, j])] <- 1 - pnorm(object$params$zeta[j, max(y[, j]) - min(y[, j])] - eta.mat[i, j])
+            if(max(y[, j]) > 2) {
+              j.levels <- 2:(max(y[, j]) - min(y[, j]))#
+              for (k in j.levels) {
+                probK[k] <- pnorm(object$params$zeta[j, k] - eta.mat[i, j]) - pnorm(object$params$zeta[j, k - 1] - eta.mat[i, j])
+              }
+            }
+            probK <- c(0, probK)
+            cumsum.b <- sum(probK[1:(y[i, j] + 2 - min(y[, j]))])
+            cumsum.a <- sum(probK[1:(y[i, j])])
+            u <- runif(n = 1, min = cumsum.a, max = cumsum.b)
+            if (abs(u - 1) < 1e-05)
+              u <- 1
+            if (abs(u - 0) < 1e-05)
+              u <- 0
+            ds.res[i, j] <- qnorm(u)
+          } else {
+            probK <- NULL
+            probK[1] <- pnorm(object$params$zeta[1] - eta.mat[i, j], log.p = FALSE)
+            probK[max(y) + 1 - min(y)] <- 1 - pnorm(object$params$zeta[max(y) - min(y)] - eta.mat[i, j])
+              levels <- 2:(max(y) - min(y))#
+              for (k in levels) {
+                probK[k] <- pnorm(object$params$zeta[k] - eta.mat[i, j]) - pnorm(object$params$zeta[k - 1] - eta.mat[i, j])
+              }
+            probK <- c(0, probK)
+            cumsum.b <- sum(probK[1:(y[i, j] + 2 - min(y))])
+            cumsum.a <- sum(probK[1:(y[i, j])])
+            u <- runif(n = 1, min = cumsum.a, max = cumsum.b)
+            if (abs(u - 1) < 1e-05)
+              u <- 1
+            if (abs(u - 0) < 1e-05)
+              u <- 0
+            ds.res[i, j] <- qnorm(u)
           }
-        }
-        probK <- c(0, probK)
-        cumsum.b <- sum(probK[1:(y[i, j] + 2 - min(y[, j]))])
-        cumsum.a <- sum(probK[1:(y[i, j])])
-        u <- runif(n = 1, min = cumsum.a, max = cumsum.b)
-        if (abs(u - 1) < 1e-05)
-          u <- 1
-        if (abs(u - 0) < 1e-05)
-          u <- 0
-        ds.res[i, j] <- qnorm(u)
+
       }
     }
   }
 
   return(list(residuals = ds.res, linpred = eta.mat))
 }
+
